# IBM_PROLOG_BEGIN_TAG 
# This is an automatically generated prolog. 
#  
#  
#  
# Licensed Materials - Property of IBM 
#  
# (C) COPYRIGHT International Business Machines Corp. 2000,2002 
# All Rights Reserved 
#  
# US Government Users Restricted Rights - Use, duplication or 
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp. 
#  
# IBM_PROLOG_END_TAG 
package ERRM_cli_utils;
#"@(#)43   1.14   src/rsct/rm/ER/cli/pm/ERRM_cli_utils.pm.perl, errmcli, rsct_rdrh, rdrht1f2 5/14/01 16:49:49"
######################################################################
#                                                                    #
# Package: ERRM_cli_utils.pm                                         #
#                                                                    #
# Description:                                                       #
#   This package contains utility/common subroutines for the         #
#   Event Response Resource Manager (ERRM) CLI commands.             #
#                                                                    #
# Subroutines Available:                                             #
#                                                                    #
#   error_exit - performs required cleanup and exits with the        #
#     appropriate ERRM CLI error exit code.                          #
#                                                                    #
#   printCIMsg - print common informational message to STDOUT.       #
#     Requires the program name and message mnemonic as inputs.      #
#                                                                    #
#   printCEMsg - print common error message to STDERR.               #
#     Requires the program name and message mnemonic as inputs.      #
#                                                                    #
#   find_resource_name - find the resource name in a specified       #
#     class to determine if it exists.                               #
#     Requires the resource name and class as inputs.                #
#                                                                    #
#   find_resource_handle - find the resource handle in the           #
#     Association class to determine if it exists.                   #
#     Requires the resource handle and original class as inputs.     #
#                                                                    #
#   find_association_handle - find the condition and response        #
#     handles in the Association class to determine if it exists.    #
#     Requires the condition and response handles as inputs.         #
#                                                                    #
#   get_handle - find the handle for a resource name.                #
#     Requires the resource name and class as inputs.                #
#                                                                    #
#   get_assoc_handles - find the Association class handles and       #
#     active flags for a condition and/or response handle.           #
#     Requires condiion handle and/or response handle.               #
#                                                                    #
#   parse_days_string - parse the day string for the response        #
#     commands.                                                      #
#     Requires the specified day string as input.                    #
#                                                                    #
#   parse_time_string - parse the time string for the response       #
#     commands.                                                      #
#     Requires the specified time string as input.                   #
#                                                                    #
#                                                                    #
# Examples:                                                          #
#   printCEMsg("EMsgCUcliBadOperand", $rsrc_class);                  #
#                                                                    #
#--------------------------------------------------------------------#
# Inputs:                                                            #
#   /usr/sbin/rsct/msgmaps/errmcli.errmcli.map - message mapping     #
#                                                                    #
# Outputs:                                                           #
#   stdout - common informational messages that get displayed.       #
#   stderr - common error messages that get displayed.               #
#                                                                    #
# External References:                                               #
#   Commands: ctdspmsg                                               #
#                                                                    #
# Tab Settings:                                                      #
#   4 and tabs should be expanded to spaces before saving this file. #
#   in vi:  (:set ts=4  and   :%!expand -4)                          #
#                                                                    #
# Change Activity:                                                   #
#   000921 JAC 67664: Initial design & write.                        #
#   001013 JAC 67469: Add varaibles for condition commands.          #
#   010102 JAC 67470: Updates for mk and rm commands.                #
#   010119 JAC 67471: Updates for start and stop commands.           #
#   010215 JAC 71203: Fix parse_days_string when in a date range.    #
#   010308 JAC 71005: Escape some special characters in strings.     #
#   010420 JAC 73421: Change get_handle to check for null handle.    #
#                     (above should have been part of 72894)         #
#   010501 JAC 73619: Do not send any RMC CLI std err to garbage.    #
#   010507 JAC 73620: Check for RMC CLI user error (rc=5) and        #
#                     return ERRM user error for this case.          #
######################################################################

use Exporter ();
@ISA = qw(Exporter);
@EXPORT_OK = qw(
    error_exit
    printCIMsg
    printCEMsg
    find_resource_name
    find_resource_handle
    find_association_handle
    get_handle
    get_assoc_handles
    parse_days_string
    parse_time_string
    $TRUE $FALSE
    $CTDIR $CTBINDIR
    $MKRSRC $RMRSRC $CHRSRC $LSRSRC $RUNACT
    $RSCCOND $RSCEVRS $RSCASSC
    $MKCNDCRIT $MKCNDWARN $MKCNDINFO
    $SEVCRIT $SEVWARN $SEVINFO $DELIMITER
    $RMC_CLI_USER_ERROR
    @EVENTTYPE @EVENTTPCD
    $GARBAGE $LSNOTFND $STARTMON $STOPMON
);

use lib "/usr/sbin/rsct/pm";
use locale;

use ERRM_cli_rc qw(ERRM_CLI_RMC_ERROR
                 ERRM_CLI_ERROR
                 ERRM_CLI_BAD_FLAG
                 ERRM_CLI_BAD_OPERAND
                 ERRM_CLI_USER_ERROR);

#--------------------------------------------------------------------#
# Global Variables                                                   #
#--------------------------------------------------------------------#
$MSGCAT = "errmcli.cat";               # msg catalogue for this cmd
$MSGSET = "errmcli";                   # common message set

$CTDIR = "/usr/sbin/rsct";             # RSCT root directory
$CTBINDIR = "$CTDIR/bin";              # Cluster Bin directory path
$LSMSG = "$CTBINDIR/ctdspmsg";         # list / display message rtn
$ENV{'MSGMAPPATH'} = "$CTDIR/msgmaps"; # msg maps used by $LSMSG

$MKRSRC = "mkrsrc";                    # RMC make resource command
$RMRSRC = "rmrsrc";                    # RMC remove resource command
$CHRSRC = "chrsrc";                    # RMC change resource command
$LSRSRC = "lsrsrc";                    # RMC list resource command
$RUNACT = "runact";                    # RMC run action command
$RSCCOND = "IBM.Condition";            # Condition Resource class name
$RSCEVRS = "IBM.EventResponse";        # Event Resp Resource class
$RSCASSC = "IBM.Association";          # Association Resource class

$LSNOTFND = 0;                         # number of last index on
                                       # lsrsrc if no condition found
$MKCNDCRIT = 2;                        # sev for mkcond critical
$MKCNDWARN = 1;                        # sev for mkcond warning
$MKCNDINFO = 0;                        # sev for mkcond informational
$SEVCRIT = "c";                        # sev for mkcond critical flag
$SEVWARN = "w";                        # sev for mkcond warning flag
$SEVINFO = "i";                        # sev for mkcond info flag
$DAYSNUM = "1234567";                  # days of the week (1=Sun)
@DAYSNUMCD = (1,2,4,8,16,32,64);       # bit map for days encoded
@EVENTTYPE = ("a","r","b");            # event types for mkresp
@EVENTTPCD = (1,2,3);                  # encoded types for mkresp
$STARTMON = 1;                         # start monitoring condition
$STOPMON = 0;                          # stop monitoring condition
$DELIMITER = "tvtv";                   # a delimiter to use
$GARBAGE = "/dev/null";                # place to put garbage
$RMC_CLI_USER_ERROR = 5;               # to check for RMC CLI user error

$FALSE = 0;
$TRUE = 1;

#--------------------------------------------------------------------#
# Exported Subroutines (with @EXPORT_OK, -> on demand).              #
#--------------------------------------------------------------------#

#--------------------------------------------------------------------#
# Common message handling (error, informational) routines:           #
#--------------------------------------------------------------------#

#--------------------------------------------------------------------#
# error_exit - performs required cleanup and exits.                  #
# Parameters:                                                        #
#   $badrc            in      Bad return code - bad enough to exit   #
#                             processing of this command.            #
# Exit:                                                              #
#   1 ERRM_CLI_RMC_ERROR      Underlying RMC error.                  #
#   2 ERRM_CLI_ERROR          Unexpected error in the command script.#
#   3 ERRM_CLI_BAD_FLAG       Input flag error.                      #
#   4 ERRM_CLI_BAD_OPERAND    Input operand error.                   #
#   5 ERRM_CLI_USER_ERROR     User error.                            #
#                                                                    #
# Global References:                                                 #
#   $main::Cleanup.    in     Hash indicating what needs to be       #
#                             cleaned up.                            #
#                             if {Session} defined - value is        #
#                             session that must terminate.           #
#                             terminate.                             #
#--------------------------------------------------------------------#
sub error_exit
{
my ($badrc) = @_;

SWITCH: {
    ($badrc == ERRM_CLI_RMC_ERROR)   && exit($badrc);
    ($badrc == ERRM_CLI_ERROR)       && exit($badrc);
    ($badrc == ERRM_CLI_BAD_FLAG)    && exit($badrc);
    ($badrc == ERRM_CLI_BAD_OPERAND) && exit($badrc);
    ($badrc == ERRM_CLI_USER_ERROR)  && exit($badrc);
    # At this point all return codes should have been converted to
    # a valid ERRM CLI return code.  But if one wasn't write an
    # error message.
    printCEMsg("EMsgERRMcliBadRC", $badrc);
    exit(ERRM_CLI_ERROR);
}   # end switch

}   # end error_exit


#--------------------------------------------------------------------#
# printCIMsg : Calls $LSMSG to print out the common cluster          #
#   ERRM CLI information messages with the required paramaters.      #
#   Messages printed to stdout.                                      #
#   This subroutine is like printIMsg except it is used to print     #
#   the common ERRM CLI messages which are in the errmcli message    #
#   set.                                                             #
#                                                                    #
# Paramaters:                                                        #
#   $msg       in  Message mnemonic / message number in a sense.     #
#   $optargs   in  Extra arguments/parameters to send to $LSMSG.     #
#                                                                    #
# Returns:  None.                                                    #
#                                                                    #
# Global Variables:                                                  #
#   $main::Trace    in  Prints extra info when trace is on.          #
#   $LSMSG          in  Path & Command to display messages.          #
#   $MSGCAT         in  MC CLI Message catalogue.                    #
#   $MSGSET         in  MC CLI common message set "mccli".           #
#--------------------------------------------------------------------#
sub printCIMsg
{
my ($msg, @optargs) = @_;
my ($optarg, $optargs);

$main::Trace &&
    print STDERR "$LSMSG $MSGSET $MSGCAT $msg @optargs\n";

# Keep the args to LSMSG separate by separating with single quotes
# but must replace internal single quotes with blanks or get an error.
# Must escape internal double quotes for the system call.
foreach $optarg (@optargs) {
    $optarg =~ s/'/ /g;
    $optarg =~ s/"/\\"/g;
}
$optargs = "'" . join("' '",@optargs) . "'";

(scalar @optargs > 0) ?
    system "$LSMSG $MSGSET $MSGCAT $msg $optargs" :
    system "$LSMSG $MSGSET $MSGCAT $msg";

return;
}   # end printCIMsg


#--------------------------------------------------------------------#
# printCEMsg : Calls $LSMSG to print out the common cluster          #
#   ERRM CLI error messages with the required paramaters.            #
#   Messages printed to stderr.                                      #
#   This subroutine is like printEMsg except it is used to print     #
#   the common ERRM CLI messages which are in the errmcli message    #
#   set and it prefixes the message with the appropriate program     #
#   name.                                                            #
#                                                                    #
# Paramaters:                                                        #
#   $msg       in  Message mnemonic / message number in a sense.     #
#   $optargs   in  Extra arguments/parameters to send to $LSMSG.     #
#                                                                    #
# Returns:  None.                                                    #
#                                                                    #
# Global Variables:                                                  #
#   $main::PROGNAME in  Calling program/command for error message.   #
#   $main::Trace    in  Prints extra info when trace is on.          #
#   $LSMSG          in  Path and command to display messages.        #
#   $MSGCAT         in  CU CLI Message catalogue.                    #
#   $MSGSET         in  CU CLI common message set "cucli".           #
#--------------------------------------------------------------------#
sub printCEMsg
{
my ($msg, @optargs) = @_;
my ($optarg, $optargs);

$main::Trace &&
    print STDERR "$LSMSG $MSGSET $MSGCAT $msg $main::PROGNAME @optargs\n";

# Keep the args to LSMSG separate by separating with single quotes
# but must replace internal single quotes with blanks or get an error.
# Must escape internal double quotes for the system call.
foreach $optarg (@optargs) {
    $optarg =~ s/'/ /g;
    $optarg =~ s/"/\\"/g;
}
$optargs = "'" . join("' '",@optargs) . "'";

(scalar @optargs > 0) ?
    system "$LSMSG $MSGSET $MSGCAT $msg $main::PROGNAME $optargs 1>&2" :
    system "$LSMSG $MSGSET $MSGCAT $msg $main::PROGNAME 1>&2";

return;
}   # end printCEMsg


#--------------------------------------------------------------------#
# find_resource_name : Uses the list resource RMC CLI command to     #
#   determine if a condition name or response name exists. It is     #
#   determined to exist if there is information returned from the    #
#   list command.                                                    #
#                                                                    #
# Paramaters:                                                        #
#   $resource_name   the condition or response name to find          #
#   $resource_class  the class to look in (Condition, EventResponse) #
#                                                                    #
# Returns:                                                           #
#   0                if the name is found                            #
#   -1               if the name is not found                        #
#   ERRM_CLI_USER_ERROR  if there was an RMC_CLI_USER_ERROR          #
#                                                                    #
# Global Variables:                                                  #
#   $main::Trace     in case trace is on                             #
#   $main::PROGNAME  for trace message                               #
#   $CTBINDIR        where list command resides                      #
#   $LSRSRC          RMC CLI list command                            #
#   $LSNOTFND        number of output line for name not found        #
#--------------------------------------------------------------------#
sub find_resource_name
{
my $resource_name = shift(@_);
my $resource_class = shift(@_);
my @listout = ();
my $trace_opt = "";
my $rc = 0;

$main::Trace &&
    print STDERR "$main::PROGNAME: calling $LSRSRC\n";
$main::Trace && ($trace_opt = " -T ");

# list resource_name using the RMC CLI list command to see if it exists
@listout = `$CTBINDIR/$LSRSRC $trace_opt -x -s "Name == \\\"$resource_name\\\"" $resource_class`;
$rc = $?;



$rc = $rc/256;
$main::Trace &&
     print STDERR "$main::PROGNAME: $LSRSRC returned $rc\n";

# if lsrsrc command failed due to RMC CLI user error   73620
# return ERRM CLI user error 
   if ($rc == $RMC_CLI_USER_ERROR) {
      return(ERRM_CLI_USER_ERROR);
      }

# if list command failed for other reason, print RMC error message and exit 
if ($rc != 0) { 
    printCEMsg("EMsgERRMcliUnExpectRMCrc",$rc);
    exit(ERRM_CLI_RMC_ERROR);
    }

# if there's output returned from the list, the resource_name exists
# set $rc accordingly
if ($#listout > $LSNOTFND) {           # resource_name found
    $rc = 0;
    }
else                       {           # resource_name not found
    $rc = -1;
    }

return ($rc);
}

#--------------------------------------------------------------------#
# find_resource_handle : Uses the list resource RMC CLI command to   #
#   determine if a condition or response handle exists in the        #
#   Association resource class.                                      #
#                                                                    #
# Paramaters:                                                        #
#   $resource_handle the condition or response handle to find        #
#   $resource_type   "c" for condition, "r" for response             #
#                                                                    #
# Returns:                                                           #
#   0                if the handle is found in the Association class #
#   -1               if the handle is not found                      #
#   ERRM_CLI_USER_ERROR  if lsrsrc returns RMC_CLI_USER_ERROR        #
#                                                                    #
# Global Variables:                                                  #
#   $main::Trace     in case trace is on                             #
#   $main::PROGNAME  for trace message                               #
#   $CTBINDIR        where list command resides                      #
#   $LSRSRC          RMC CLI list command                            #
#   $RSCASSC         Association class                               #
#   $LSNOTFND        number of output line for name not found        #
#--------------------------------------------------------------------#
sub find_resource_handle
{
my $resource_handle = shift(@_);
my $resource_type = shift(@_);
my @listout = ();
my $trace_opt = "";
my $rc = 0;

$main::Trace &&
    print STDERR "$main::PROGNAME: calling $LSRSRC\n";
$main::Trace && ($trace_opt = " -T ");

# list resource_name using the RMC CLI list command to see if it exists
if ($resource_type eq "c") {
   @listout = `$CTBINDIR/$LSRSRC $trace_opt -x -s 'ConditionHandle==$resource_handle' $RSCASSC`;
   $rc = $?;
}
else {
   @listout = `$CTBINDIR/$LSRSRC $trace_opt -x -s 'EventResponseHandle==$resource_handle' $RSCASSC`;
   $rc = $?;
}

$rc = $rc/256;
$main::Trace &&
    print STDERR "$main::PROGNAME: $LSRSRC returned $rc\n";
# if lsrsrc command failed due to RMC CLI user error   73620
# return ERRM CLI user error
   if ($rc == $RMC_CLI_USER_ERROR) {
      return(ERRM_CLI_USER_ERROR);
      }

# if list command failed for other reason, print RMC error message and exit
if ($rc != 0) {
    printCEMsg("EMsgERRMcliUnExpectRMCrc",$rc);
    exit(ERRM_CLI_RMC_ERROR);
    }

# if there's output returned from the list, the resource_handle exists
# set $rc accordingly
if ($#listout > $LSNOTFND) {           # resource_handle found
    $rc = 0;
    }
else                       {           # resource_handle not found
    $rc = -1;
    }

return ($rc);
}


#--------------------------------------------------------------------#
# find_association_handle : Uses the list resource RMC CLI command   #
#   to determine if a condition/response handle exists in the        #
#   Association resource class.                                      #
#                                                                    #
# Paramaters:                                                        #
#   $cond_handle     the condition handle                            #
#   $resp_handle     the response handle                             #
#                                                                    #
# Returns:                                                           #
#   0                if the condition/response handles are found     # 
#                    in the Association class                        # 
#   -1               if the handle is not found                      #
#   ERRM_CLI_USER_ERROR   if RMC_CLI_USER_ERROR                      #
#                                                                    #
# Global Variables:                                                  #
#   $main::Trace     in case trace is on                             #
#   $main::PROGNAME  for trace message                               #
#   $CTBINDIR        where list command resides                      #
#   $LSRSRC          RMC CLI list command                            #
#   $RSCASSC         Association class                               #
#   $LSNOTFND        number of output line for name not found        #
#--------------------------------------------------------------------#
sub find_association_handle
{
my $cond_handle = shift(@_);
my $resp_handle = shift(@_);
my @listout = ();
my $trace_opt = "";
my $rc = 0;

$main::Trace &&
    print STDERR "$main::PROGNAME: calling $LSRSRC\n";
$main::Trace && ($trace_opt = " -T ");

# check to see if the condition - resource are already in Association
@listout = `$CTBINDIR/$LSRSRC $trace_opt -x -s 'ConditionHandle==$cond_handle & EventResponseHandle==$resp_handle' $RSCASSC`;
$rc = $?;

$rc = $rc/256;
$main::Trace &&
     print STDERR "$main::PROGNAME: $LSRSRC returned $rc\n";

# if lsrsrc command failed due to RMC CLI user error   73620
# return ERRM CLI user error
if ($rc == $RMC_CLI_USER_ERROR) {
   return(ERRM_CLI_USER_ERROR);
   }

# if list command failed for other reason, print RMC error message and exit
if ($rc != 0) {
    printCEMsg("EMsgERRMcliUnExpectRMCrc",$rc);
    exit(ERRM_CLI_RMC_ERROR);
    }

# if there's output returned from the list, the resource_handle exists
# set $rc accordingly
if ($#listout > $LSNOTFND) {           # handles found
    $rc = 0;
    }
else                       {           # handles not found
    $rc = -1;
    }

return ($rc);
}


#--------------------------------------------------------------------#
# get_handle: Obtain the resource handle for the named resource in   #
#   the class given.  Uses the RMC CLI lsrsrc command.               #
#                                                                    #
# Paramaters:                                                        #
#   $resource_name   the existing condition or response name         #
#   $resource_class  the class to look in (Condition, EventResponse) #
#                                                                    #
# Returns:                                                           #
#   $rc              0 if resource handle found                      #
#                    ERRM_CLI_USER_ERROR if RMC_CLI_USER_ERROR       #
#   $resource_handle the resource handle                             #
#                                                                    #
# Global Variables:                                                  #
#   $main::Trace     in case trace is on                             #
#   $main::PROGNAME  for trace message                               #
#   $CTBINDIR        where list command resides                      #
#   $LSRSRC          RMC CLI list command                            #
#--------------------------------------------------------------------#
sub get_handle
{
my $resource_name = shift(@_);
my $resource_class = shift(@_);
my $resource_handle = "";
my $rc = 0;

$main::Trace &&
    print STDERR "$main::PROGNAME: calling $LSRSRC\n";

# list resource_name using the RMC CLI list command to get the handle
$resource_handle = `$CTBINDIR/$LSRSRC -t -x -s "Name == \\\"$resource_name\\\"" $resource_class ResourceHandle`;
$rc = $?;
$rc = $rc/256;
$main::Trace &&
    print STDERR "$main::PROGNAME: $LSRSRC returned $rc\n";

# if lsrsrc command failed due to RMC CLI user error   73620
# return ERRM CLI user error
   if ($rc == $RMC_CLI_USER_ERROR) {
      return(ERRM_CLI_USER_ERROR);
      }
# if list command failed, print RMC error message and exit
if ($rc != 0) {
    printCEMsg("EMsgERRMcliUnExpectRMCrc",$rc);
    exit(ERRM_CLI_RMC_ERROR);
    }

if ($resource_handle eq "") {          # handle not found
    $rc = -1;
    }
else                       {           # handle found
    chomp($resource_handle);
    }

return ($rc,$resource_handle);
}

#--------------------------------------------------------------------#
# parse_days_string - Parse the days of the week string returning    #
#   the encoded days for the Actions attribute of EventResponse.     #
#   The default of 1-7 is filled in if no value is specified (for    #
#   example: 3,4,,8.)                                                #
#                                                                    #
# Input:                                                             #
#   $days_string             valid user input for -d                 #
# Return:                                                            #
#   $ndays                   the number of coded days found          #
#   $days_code               the coded days value                    #
#                                                                    #
#--------------------------------------------------------------------#
sub parse_days_string
{
my $days_string = shift @_;             # input days string
my $ndays = 0;                          # count the number of coded days
my $days_code = "";                     # will return coded days
my $day = "";                           # part of $day_string
my $lendays = 0;                        # length of $days_string
my $range = $FALSE;                     # looks for a - in $days_string
my $tot_code = 0;                       # keeps count of coded days
my $i = 0;                              # loop counter
my $k = 0;                              # loop counter
my $pos = 0;                            # find coded values
my $range_num = 0;                      # next number for range
my $comma = "";                         # when >1 days, need a comma


# look for any ,, used.  Fill them in with 1-7 (default)
while($days_string =~ /,,/) { $days_string =~ s/,,/,1-7,/;}

# find the length of day string
$lendays = length $days_string;

# look for an ending comma.  Fill in with 1-7 (default)
if (substr($days_string,$lendays-1,1) eq ",") {
   $days_string = $days_string . "1-7";
   }

# look for a beginning comma. Fill in with 1-7 (default)
if (substr($days_string,0,1) eq ",") {
   $days_string = "1-7" . $days_string;
   }

# make sure we have the corrected length of day string
$lendays = length $days_string;

# loop looking at each character in day string.
# make sure it's 1-7, a -, or a comma and process accordingly.
for ($i=0;$i<$lendays;$i++){
    $day = substr($days_string,$i,1);
 
    # is day character a number 1 to 7?
    if ($day =~ /[1-7]/){
       # it's a number. are we in a range that was specified?
       if ($range){
          # it's in a range.  is the next number in the range valid?
          if ($day lt $range_num) {                   # 71203 lt, not le
              printCEMsg("EMsgERRMcliImproperUsageOperand","-d");
              exit(ERRM_CLI_USER_ERROR);
             }

          # in a valid range, add days until we come to the end of the range
          for ($k=$range_num;$k<=$day;$k++){
              $pos = index $DAYSNUM,$k;       # determine coded value
              $tot_code = $tot_code + $DAYSNUMCD[$pos];  # add to total
              }
          $range = $FALSE;              # reset back after processing done
       }

       # it's a number, but not part of a range
       else {
           $pos = index $DAYSNUM,$day;  # determine coded value 
           $tot_code = $tot_code + $DAYSNUMCD[$pos];  # add to total
           }
 
       # in case this is the start of a range, keep track of what should be next
       $range_num = $day + 1;
       }                                # end of if it's 1-7

    # is day character a hyphen?
    elsif ($day eq "-") { 

       # if we're in a range, it can't be a hyphen
       if ($range) { 
           printCEMsg("EMsgERRMcliImproperUsageOperand","-d");
           exit(ERRM_CLI_USER_ERROR);
           }
       # set the range flag so we know we're in a range specification
       $range = $TRUE; 
       }                                # end of it's a hyphen

    # is day character a plus? (multiple days but not a range)
    elsif ($day eq "+") {

       # don't need to do anything.  move on to next day .
       }                                # end of it's a plus

    # is day character a comma?
    elsif ($day eq ",") {

       # if we're in a range, it can't be a comma
       if ($range) {
           printCEMsg("EMsgERRMcliImproperUsageOperand","-d");
           exit(ERRM_CLI_USER_ERROR);
           }

       # end of previous day specification. concatenate to coded string and
       # reset some things
       $days_code = $days_code . $comma . $tot_code;
       $tot_code = 0;
       $range = $FALSE;
       $comma = ",";
       $ndays++;                        # count the number of coded values
       }                                # end of it's a comma

    # an invalid character 
    else {
       printCEMsg("EMsgERRMcliImproperUsageOperand","-d");
       exit(ERRM_CLI_USER_ERROR);
       }
}

# add the last coded value to the coded value string
$days_code = $days_code . $comma . $tot_code;

$ndays++;                               # count the last value 

return ($ndays,$days_code);
}


#--------------------------------------------------------------------#
# parse_time_string - Parse the times of the day string returning    #
#   the encoded times for the Actions attribute of EventResponse.    #
#   The default of 0000-2400 is filled in if no value is specified   #
#   (for example: 0800-1600,,0800-1800)                              #
#                                                                    #
# Input:                                                             #
#   $time_string             valid user input for -t                 #
# Return:                                                            #
#   $ntimes                  the number of coded times found         #
#   $start_time              the coded start times                   #
#   $end_time                the coded end times                     #
#                                                                    #
#--------------------------------------------------------------------#
sub parse_time_string
{
my $time_string = shift @_;             # input times string
my $time_code = 0;                      # coded time
my $ntimes = 0;                         # number of coded times
my $start_time = "";                    # coded start times
my $end_time = "";                      # coded end times
my $this_is_start_time = $TRUE;         # flag to know what time
my $hhmm = "";                          # substring of time_string  
my $hours = "";                         # substring of hhmm
my $minutes = "";                       # substring of hhmm 
my $lentime = 0;                        # length of time_string
my $i = 0;                              # loop counter
my $prev_start_time = 0;                # remember start time
my $comma = "";                         # separates multiple times

# look for ending any comma.  Fill in with 0000-2400 (default)
while($time_string =~ /,,/) { $time_string =~ s/,,/,0000-2400,/; }

# get the length of the time string
$lentime = length $time_string;

# look for ending comma.  Fill in with 0000-2400 (default)
if (substr($time_string,$lentime-1,1) eq ",") {
   $time_string = $time_string . "0000-2400";
   }

# look for beginning comma.  Fill in with 0000-2400 (default)
if (substr($time_string,0,1) eq ",") {
   $time_string = "0000-2400" . $time_string;
   }

# get possibly new length
$lentime = length $time_string;

# loop through time string pulling off start and stop time.
# validate the times and convert to coded times
for ($i=0;$i<$lentime;$i=$i+5){

    if ($i>=10) { $comma = ",";}        # have >1 time specified

    # check the time format (hh:mm). enough characters?
    if ( ($i+3)>($lentime-1) ) {
         printCEMsg("EMsgERRMcliImproperUsageOperand","-t");
         exit(ERRM_CLI_USER_ERROR);
         }

    # get a time
    $hhmm = substr($time_string,$i,4);

    # extract the hours and check them
    $hours = substr($hhmm,0,2);
    if ($hours lt "00" | $hours gt "24")  {
        printCEMsg("EMsgERRMcliImproperUsageOperand","-t");
        exit(ERRM_CLI_USER_ERROR);
        }

    # extract the times and check them
    $minutes = substr($hhmm,2,2);
    if ($minutes lt "00" | $minutes gt "59")  {
        printCEMsg("EMsgERRMcliImproperUsageOperand","-t");
        exit(ERRM_CLI_USER_ERROR);
        }

    # calculate the coded time in seconds
    $time_code = ($hours * 60 * 60) + ($minutes * 60);

    # for start time, add to coded start times and save to compare
    # against the end time
    if ($this_is_start_time){
       $this_is_start_time = $FALSE;
       $start_time = $start_time . $comma . $time_code;
       $prev_start_time = $time_code;
    }

    # for end time, verify it's more than start time, then add to 
    # coded end times.
    else {
       if ($prev_start_time gt $time_code) {
           printCEMsg("EMsgERRMcliImproperUsageOperand","-t");
           exit(ERRM_CLI_USER_ERROR);
           }
       $this_is_start_time = $TRUE;
       $end_time = $end_time . $comma . $time_code;
       $ntimes++;                       # count the number of times 
       }
    }

return ($ntimes,$start_time,$end_time);
}


#--------------------------------------------------------------------#
# get_assoc_handles : Uses the list resource RMC CLI command to      #
#   find all association resource handles from a condition handle    #
#   and / or a response handle in the Association resource class.    #
#                                                                    #
# Paramaters:                                                        #
#   $cond_handle     the condition handle to find, or ""             #
#   $resp_handle     the response handle to find, or ""              #
#                                                                    #
# Returns:                                                           #
#   $rc              return code                                     #
#                    0  if one or more Association handles are found #
#                    -1 if no Association handles are found          #
#                    ERRM_CLI_USER_ERROR if RMC_CLI_USER_ERROR       #
#   \@handles        all Association handles that were found         #
#                    (by reference)                                  #
#   \@actflags       the corresponding ActiveFlag values             #
#                    (by reference)                                  #
#                                                                    #
# Global Variables:                                                  #
#   $main::Trace     in case trace is on                             #
#   $main::PROGNAME  for trace message                               #
#   $CTBINDIR        where list command resides                      #
#   $LSRSRC          RMC CLI list command                            #
#   $RSCASSC         Association class                               #
#   $LSNOTFND        number of output line for name not found        #
#--------------------------------------------------------------------#
sub get_assoc_handles
{
my $cond_handle = shift(@_);            # condition resource handle
my $resp_handle = shift(@_);            # response resource handle
my $rc = 0;                             # return code
my @handles = ();                       # array to keep handles
my @act_flags = ();                     # array to keep active flags
my @listout = ();                       # lsrsrc output
my $i = 0;                              # loop counter
my $pos = 0;                            # look through output
my $handle = "";                        # a single handle from output
my $active = 0;                         # a single flag from output
my $trace_opt = "";

$main::Trace &&
    print STDERR "$main::PROGNAME: calling $LSRSRC\n";
$main::Trace && ($trace_opt = " -T ");

# list the Association resource depending on whether you have just a condition,
# just a response, or both
if ($resp_handle eq "") {               # have only a condition
   @listout = `$CTBINDIR/$LSRSRC $trace_opt -x -s 'ConditionHandle==$cond_handle' $RSCASSC ResourceHandle ActiveFlag`;
   $rc = $?;
   }

elsif ($cond_handle eq "") {            # have only a response
   @listout = `$CTBINDIR/$LSRSRC $trace_opt -x -s 'EventResponseHandle==$resp_handle' $RSCASSC ResourceHandle ActiveFlag`;
   $rc = $?;
   }

else {                                  # have both condition and response
   @listout = `$CTBINDIR/$LSRSRC $trace_opt -x -s 'ConditionHandle==$cond_handle & EventResponseHandle==$resp_handle' $RSCASSC ResourceHandle ActiveFlag`;
   $rc = $?;
   }

$rc = $rc/256;
$main::Trace &&
     print STDERR "$main::PROGNAME: $LSRSRC returned $rc\n";

# if lsrsrc command failed due to RMC CLI user error   73620
# return ERRM CLI user error
if ($rc == $RMC_CLI_USER_ERROR) {
   return(ERRM_CLI_USER_ERROR);
   }

# if list command failed for other reason, print RMC error message and exit
if ($rc != 0) {
    printCEMsg("EMsgERRMcliUnExpectRMCrc",$rc);
    exit(ERRM_CLI_RMC_ERROR);
    }

# if there's output returned from the list, the resource_handle exists
# set $rc accordingly
if ($#listout > $LSNOTFND) {           # resource_handle found
    $rc = 0;
    }
else                       {           # resource_handle not found
    $rc = -1;
    }

# parse the output to find the handles and the active flags
for ($i=0;$i<=$#listout;$i++) {        # each line of output

   # does this line have a handle?
   $pos = index $listout[$i],"ResourceHandle";
   if ($pos>=0) {                      # yes, there's a handle
      $pos = index $listout[$i]," = "; # find where it starts
      $handle = substr($listout[$i],$pos+3);
      chomp($handle);
      push @handles, $handle;          # save the handle
      }

   # does this line have an active flag?
   else {
      $pos = index $listout[$i],"ActiveFlag";
      if ($pos>=0) {                   # yes, there's a flag
         $pos = index $listout[$i]," = "; # find where it starts
         $active = substr($listout[$i],$pos+3);
         chomp($active);
         push @act_flags, $active;     # save the flag
         }
      }
  }

# return the arrays back as references
return ($rc, \@handles, \@act_flags);
}


#--------------------------------------------------------------------#
# End Exported Subroutines (with @EXPORT_OK, -> on demand).          #
#--------------------------------------------------------------------#


#--------------------------------------------------------------------#
# End File.                                                          #
#--------------------------------------------------------------------#
